@@ -98,15 +98,13 @@ test_that("get_ests_bmlmi", {
 
     res <- get_ests_bmlmi(ests, D)
 
-    expect_true(
-        is.list(res) &
-            length(res) == 3 &
-            all(!is.na(res)) &
-            all(!is.null(res)) &
-            all(sapply(res, is.numeric)) &
-            all(sapply(res, function(x) length(x) == 1)) &
-            res$est_point == mean(ests)
-    )
+    expect_true(is.list(res))
+    expect_length(res, 3)
+    expect_true(all(!is.na(res)))
+    expect_true(all(!is.null(res)))
+    expect_true(all(sapply(res, is.numeric)))
+    expect_true(all(sapply(res, function(x) length(x) == 1)))
+    expect_equal(round(res$est_point, 4), round(mean(ests), 4))
 
     D <- 3
     expect_error(
